import os, netCDF4, cfgrib, zipfile, shutil, json
import numpy as np
import xarray as xr
import datetime
from pathlib import Path
from types import SimpleNamespace

this_dir = os.path.dirname(__file__)
dirs = SimpleNamespace(
            test_dir = this_dir,
            nc_file=os.path.join(this_dir, "sample.nc"),
            grib_file=os.path.join(this_dir, "sample.grib"),
            zip_file = os.path.join(this_dir, "sample.zip"),
            log_dir=os.path.join("/tmp", "dqc_logs", "test"),
            constraints_json = os.path.join(this_dir, "constraints.json")
        )

def pytest_configure():
    Path(dirs.log_dir).mkdir(parents=True, exist_ok=True)
    print("Creating fake datasets (netcdf, grib and zip formats)")
    mock_nc()
    mock_grib()
    mock_zip()
    mock_constraints()


def pytest_unconfigure():
    print("Removing fake files")
    Path(dirs.nc_file).unlink()
    Path(dirs.grib_file).unlink()
    Path(dirs.zip_file).unlink()
    Path(dirs.constraints_json).unlink()
    shutil.rmtree(dirs.log_dir)


def mock_nc():
    ds = None
    try:
        # Mocked dimension/variables
        lon = np.linspace(0, 360, 320, endpoint=False)
        lat = np.linspace(-90, 90, 160, endpoint=False)
        tas = np.random.randint(10, 25,
                              size=(1, len(lon), len(lat)))
        fake_date = datetime.date.today()
        fake_dates = [fake_date + datetime.timedelta(days=i) for i in range(5)]
        num_dates = np.array([d.toordinal() for d in fake_dates])

        # Writing netcDF4 and adding mocks
        ds = netCDF4.Dataset(dirs.nc_file, 'w', format='NETCDF4')
        ds.set_ncstring_attrs(True)
        ds.createDimension('lon', len(lon))
        ds.createDimension('lat', len(lat))
        ds.createDimension('time', None)
        ds.createDimension('tas', len(tas))
        longitude = ds.createVariable('lon', 'f4', 'lon')
        latitude = ds.createVariable('lat', 'f4', 'lat')
        time = ds.createVariable('time', 'f4', 'time')
        temp = ds.createVariable('tas', 'f4', ('time', 'lon', 'lat'))
        longitude[:] = lon
        latitude[:] = lat
        temp[:] = tas
        time[:5] = num_dates

        # Adding attributes
        ds.name = "Example dataset"
        ds.description = "Example dataset"
        longitude.units = 'degrees east'
        latitude.units = 'degrees north'
        time.units = 'days since 1-01-01'
        temp.units = 'Kelvin'
        temp.warning = 'This data is not real!'
    finally:
        ds.close()


def mock_grib():
   ds = xr.open_dataset(dirs.nc_file)
   ds = ds.isel(time=0)
   cfgrib.to_grib(ds, dirs.grib_file,
                  grib_keys={"gridType": "unstructured_grid"})

def mock_zip():
    zip = zipfile.ZipFile(dirs.zip_file, "w", zipfile.ZIP_DEFLATED)
    zip.write(dirs.nc_file, os.path.basename(dirs.nc_file))
    zip.write(dirs.grib_file, os.path.basename(dirs.grib_file))
    zip.close()

def mock_constraints():
    fd = datetime.date.today()
    constraints = [{
        "variable": ["tas"],
        "year": [fd.year],
        "month": [fd.month],
        "day": [fd.day, (fd.day)%30+1]
    }]
    with open(Path(dirs.test_dir, 'constraints.json'), 'w') as f:
        json.dump(constraints, f)

if __name__ == "__main__":
    mock_nc()
    mock_grib()
    mock_zip()
    mock_constraints()
